/********************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
 /* ******************************************************************************
  * @file    Project/STM32F4xx_StdPeriph_Template/main.c 
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    13/06/2011
  * @brief   Main program body
  ******************************************************************************
  * @copy
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2010 STMicroelectronics</center></h2>
  */ 

/* Includes ------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <yfuns.h>
#include "includes.h"

#include "stm32f4xx_gpio.h"
#include "arm_comm.h"
#include "includes.h"
#include "mmc.h"
#include "rtc.h"

#include  "usbd_hid_core.h"
#include  "usbd_usr.h"
#include  "usbd_desc.h"

#define DLY_100US  1000



void Delay_ (unsigned long a) { while (--a!=0); }


Int32U CriticalSecCntr;
/** @addtogroup Template_Project
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
USART_InitTypeDef USART_InitStructure;

/* Private function prototypes -----------------------------------------------*/
char UART_Receive (void);
void DelayResolution100us(Int32U Dely);
static int MyLowLevelGetchar(void);
size_t __write(int Handle, const unsigned char * Buf, size_t Bufsize);
size_t __read(int handle, unsigned char * buffer, size_t size);

/* Private functions ---------------------------------------------------------*/
char UART_Receive (void)
{
  if(EVAL_COM1->SR & USART_FLAG_RXNE)
    return USART_ReceiveData(EVAL_COM1);
  else
    return 0;
}



/*-------------SysTickStart---------------------------------------------------*/
void SysTickStart(uint32_t Tick)
{
RCC_ClocksTypeDef Clocks;
volatile uint32_t dummy;

  RCC_GetClocksFreq(&Clocks);

  dummy = SysTick->CTRL;  
  SysTick->LOAD = (Clocks.HCLK_Frequency/8)/Tick;
  
  SysTick->CTRL = 1;
}
/*-------------SysTickStop----------------------------------------------------*/
void SysTickStop(void)
{
    SysTick->CTRL = 0;
}

void UART_Init (void)
{
  USART_InitStructure.USART_BaudRate = 115200;
  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
  USART_InitStructure.USART_StopBits = USART_StopBits_1;
  USART_InitStructure.USART_Parity = USART_Parity_No;
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
  STM_EVAL_COMInit(COM1, &USART_InitStructure);
}

void LED_Button_Init ()
{
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA | RCC_AHB1Periph_GPIOC, ENABLE);

  // Led PC12 as output
  STM_EVAL_LEDInit (LED1);
  /*Init Buttons*/
  STM_EVAL_PBInit(BUTTON_WAKEUP, BUTTON_MODE_GPIO);
  return;
}

void Button_Click (void)
{
  while(GPIO_ReadInputDataBit(WAKEUP_BUTTON_GPIO_PORT, WAKEUP_BUTTON_PIN) == Bit_RESET);
  while(!GPIO_ReadInputDataBit(WAKEUP_BUTTON_GPIO_PORT, WAKEUP_BUTTON_PIN) == Bit_RESET);
}

int MMC_Demo (void)
{
  int Result, Error = 0;
  printf("Please insert the card with write protect ON\r\n");
  printf("Or press ESCAPE to skip the Demo!\r\n");
  do
  {
    Result = DemoMMC();
    if (UART_Receive() == 0x1B)
    {
       printf ("Card Demo skipped!\r\n");
       return 1;
    }
  }
  while (Result == 2);
  if (Result == 3)
    printf ("Write protect pin OK!\r\n");
  else
  {
    printf ("Write protect pin FAULT!\r\n");
    Error = 1;
  }
  
  printf ("Insert the card with write protect OFF and press button!\r\n");
  Button_Click ();
  do
  {
    Result = DemoMMC();
  }
  while (Result == 2);
  
  if (Result == 0)
     printf ("Write/Read communication OK!\r\n");
  else
  {
     printf ("Write/Read communication FAULT!\r\n");
     Error = 1;
  }
  printf ("Remove the card and press button again\r\n");
  Button_Click ();
  return Error;
}

int RTC_Demo(void)
{
  #define TIME  4
  RTC_TimeTypeDef time =
  {
    .RTC_Hours = 0,
    .RTC_Minutes = 0,
    .RTC_Seconds = 0,
    .RTC_H12 = 0
  };
  printf ("To start RTC Demo press the button!\r\n");
  Button_Click ();
 
  /*SysTick to times per second*/
  SysTickStart(1);

  /* Enable LSE */
  RCC_APB1PeriphClockCmd(  RCC_APB1Periph_PWR, ENABLE);
  PWR_BackupAccessCmd(ENABLE);
  /* */
  RCC_LSEConfig(RCC_LSE_ON);
  /* Wait till LSE is ready */
  int Tick = 0;
  
  while (RCC_GetFlagStatus(RCC_FLAG_LSERDY) == RESET)
  {
    /*Check Ticks*/
    if( SysTick->CTRL & (1<<16))
    {
      if(6 < ++Tick)
      {
        return 1;
      }
    }
  }
  /* Select LSE as RTC Clock Source */
  RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);

  /* Enable RTC Clock */
  RCC_RTCCLKCmd(ENABLE);

  /* Wait until last write operation on RTC registers has finished */
  RTC_WaitForSynchro();

  /*Set time*/
  RTC_SetTime(RTC_Format_BIN,&time);
  /*Convert RTC to hour min and sec*/

  Tick = 0;

  do
  {
    /*Check Ticks*/
    if( SysTick->CTRL & (1<<16))
    {
      Tick ++;
      printf("\fTime %.2d:%.2d:%.2d\r\n",time.RTC_Hours,time.RTC_Minutes,time.RTC_Seconds);

      /*Print RTC*/
      RTC_GetTime(RTC_Format_BIN,&time);
    }
  }
  while(GPIO_ReadInputDataBit(WAKEUP_BUTTON_GPIO_PORT, WAKEUP_BUTTON_PIN) == Bit_RESET);
  return 0;
}

#ifdef USB_OTG_HS_INTERNAL_DMA_ENABLED
  #if defined ( __ICCARM__ ) /*!< IAR Compiler */
    #pragma data_alignment=4   
  #endif
#endif /* USB_OTG_HS_INTERNAL_DMA_ENABLED */
__ALIGN_BEGIN USB_OTG_CORE_HANDLE  USB_OTG_dev __ALIGN_END;

#define MOUSE_DELTA  5
#define Buttons buf[0]
#define X  buf[1]
#define Y  buf[2]
void USB_Mouse(void)
{
  unsigned char buf[4] = {0, 0, 0, 0};
  unsigned int Direction=0, i=0;
  unsigned int Timeout = 0;
  
  printf ("Connect USB to move the mouse cursor and press the button!\r\n");
  Button_Click ();
  printf ("Press the button to stop!\r\n");

  while (!STM_EVAL_PBGetState (BUTTON_WAKEUP))
  {
      if (!Timeout)
      {
         STM_EVAL_LEDToggle (LED1);
         Timeout = 30;
      }
      else
         Timeout--;
      switch (Direction)
      {
      case 0:
           X += MOUSE_DELTA;
           i++;
           Direction = (Direction + (i>>5)) & 3;
           i = i & 0x1F;
        break;
      case 1:
          Y += MOUSE_DELTA;
           i++;
           Direction = (Direction + (i>>5)) & 3;
           i = i & 0x1F;
        break;
      case 2:
          X -= MOUSE_DELTA;
           i++;
           Direction = (Direction + (i>>5)) & 3;
           i = i & 0x1F;
        break;
      case 3:
          Y -= MOUSE_DELTA;
           i++;
           Direction = (Direction + (i>>5)) & 3;
           i = i & 0x1F;
        break;
      default:
        break;
      }

      if(Y || X)
      {
        // Send report
        USBD_HID_SendReport (&USB_OTG_dev, buf, 4);
        Y = X = 0;
      }
  }
  while (STM_EVAL_PBGetState (BUTTON_WAKEUP));
  return;
}

int USB_Demo (void)
{
  USBD_Init(&USB_OTG_dev, USB_OTG_FS_CORE_ID, &USR_desc, &USBD_HID_cb, &USR_cb);
  USB_Mouse ();
  return 0;
}

/*************************************************************************
 * Function Name: main
 * Parameters: none
 * Return: Int32U
 *
 * Description: The main subroutine
 *
 *************************************************************************/
int main(void)
{
  // Init clock system  
  SystemInit();

  LED_Button_Init ();
  UART_Init ();

  printf("\fOlimex STM32-P405 Demo\r\n");

  MMC_Demo ();
  RTC_Demo ();
  USB_Demo ();
  while (1)
    if (GPIO_ReadInputDataBit(WAKEUP_BUTTON_GPIO_PORT, WAKEUP_BUTTON_PIN) == Bit_RESET)
      STM_EVAL_LEDOn (LED1);
    else
      STM_EVAL_LEDOff (LED1);
}



/*************************************************************************
 * Function Name: DelayResolution100us
 * Parameters: Int32U Dly
 *
 * Return: none
 *
 * Description: Delay ~ (arg * 100us)
 *
 *************************************************************************/
void DelayResolution100us(Int32U Dely)
{
  for(; Dely; Dely--)
  {
    for(volatile Int32U j = DLY_100US; j; j--)
    {
    }
  }
}

/*************************************************************************
 * Function Name: __write
 * Parameters: Low Level character output
 *
 * Return:
 *
 * Description:
 *
 *************************************************************************/
size_t __write(int Handle, const unsigned char * Buf, size_t Bufsize)
{
size_t nChars = 0;

  for (/*Empty */; Bufsize > 0; --Bufsize)
  {
    /* Loop until the end of transmission */
    while (USART_GetFlagStatus(EVAL_COM1, USART_FLAG_TXE) == RESET);
    USART_SendData(EVAL_COM1, * Buf++);
    ++nChars;
  }
  return nChars;
}

/*************************************************************************
 * Function Name: __read
 * Parameters: Low Level character input
 *
 * Return:
 *
 * Description:
 *
 *************************************************************************/
size_t __read(int handle, unsigned char * buffer, size_t size)
{
  int nChars = 0;

  /* This template only reads from "standard in", for all other file
   * handles it returns failure. */
  if (handle != _LLIO_STDIN)
  {
    return _LLIO_ERROR;
  }

  for (/* Empty */; size > 0; --size)
  {
    int c = MyLowLevelGetchar();
    if (c < 0)
      break;

    *buffer++ = c;
    ++nChars;
  }

  return nChars;
}

static int MyLowLevelGetchar(void)
{
int ch;
unsigned int status = EVAL_COM1->SR;

  if(status & USART_FLAG_RXNE)
  {
    ch = USART_ReceiveData(EVAL_COM1);
    if(status & (USART_FLAG_ORE | USART_FLAG_PE | USART_FLAG_FE) )
    {
      return (ch | 0x10000000);
    }
    return (ch & 0xff );
  }
  return -1;
}
/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/

